import 'package:freezed_annotation/freezed_annotation.dart';

part 'platform_options.freezed.dart';
part 'platform_options.g.dart';

abstract class PlatformSecurityOptions {}

@freezed
class IosSecurityOptions extends PlatformSecurityOptions
    with _$IosSecurityOptions {
  /// [IosSecurityOptions] class containing options for iOs
  ///
  /// It enable you to pass:
  ///
  /// [bundleId] is the app bundle identifier of the app to be signed. It's required for [amITampered] and [hasBundleBeenCompromised] methods.
  ///
  /// [mobileProvision] is required for [amITampered] method.
  ///
  /// [jsonFileName] is the json name previously generated by the [script](https://github.com/ziomarco/mobile-security-hashgenerator) containing paths and corresponding MD5 datas. Required for 'hasBundleBeenCompromised' method.
  ///
  /// [cryptographicKey] is the key used to encrypt the json. If you're using the script attached in the [readme](https://github.com/ziomarco/mobile-security-hashgenerator/blob/master/README.md) the default length is 32 characters. Required for [hasBundleBeenCompromised] method.
  ///
  /// [listOfPaths] rapresents a list of the paths to get MD5 of. Required for [hasBundleBeenCompromised] method.
  factory IosSecurityOptions({
    required final String bundleId,
    final String? mobileProvision,
    final String? jsonFileName,
    final String? cryptographicKey,
    final List<String>? listOfPaths,
  }) = _IosSecurityOptions;

  factory IosSecurityOptions.fromJson(Map<String, dynamic> json) =>
      _$IosSecurityOptionsFromJson(json);
}

@freezed
class AndroidSecurityOptions extends PlatformSecurityOptions
    with _$AndroidSecurityOptions {
  factory AndroidSecurityOptions({final String? sha1}) =
      _AndroidSecurityOptions;
  factory AndroidSecurityOptions.fromJson(Map<String, dynamic> json) =>
      _$AndroidSecurityOptionsFromJson(json);
}
